@@ -35,6 +35,11 @@ extern "C" {
 #include <spa/pod/iter.h>
 #include <spa/pod/vararg.h>
 
+/**
+ * \addtogroup spa_pod
+ * \{
+ */
+
 struct spa_pod_parser_state {
 	uint32_t offset;
 	uint32_t flags;
@@ -48,7 +53,7 @@ struct spa_pod_parser {
 	struct spa_pod_parser_state state;
 };
 
-#define SPA_POD_PARSER_INIT(buffer,size)  (struct spa_pod_parser){ buffer, size, }
+#define SPA_POD_PARSER_INIT(buffer,size)  (struct spa_pod_parser){ buffer, size, 0, {} }
 
 static inline void spa_pod_parser_init(struct spa_pod_parser *parser,
 				       const void *data, uint32_t size)
@@ -78,7 +83,7 @@ static inline struct spa_pod *
 spa_pod_parser_deref(struct spa_pod_parser *parser, uint32_t offset, uint32_t size)
 {
 	if (offset + 8 <= size) {
-		struct spa_pod *pod = SPA_MEMBER(parser->data, offset, struct spa_pod);
+		struct spa_pod *pod = SPA_PTROFF(parser->data, offset, struct spa_pod);
 		if (offset + SPA_POD_SIZE(pod) <= size)
 			return pod;
 	}
@@ -87,7 +92,7 @@ spa_pod_parser_deref(struct spa_pod_parser *parser, uint32_t offset, uint32_t si
 
 static inline struct spa_pod *spa_pod_parser_frame(struct spa_pod_parser *parser, struct spa_pod_frame *frame)
 {
-	return SPA_MEMBER(parser->data, frame->offset, struct spa_pod);
+	return SPA_PTROFF(parser->data, frame->offset, struct spa_pod);
 }
 
 static inline void spa_pod_parser_push(struct spa_pod_parser *parser,
@@ -398,10 +403,13 @@ do {											\
 	case 'T':									\
 	case 'O':									\
 	case 'V':									\
-		*va_arg(args, const struct spa_pod**) =					\
-			(pod == NULL || (SPA_POD_TYPE(pod) == SPA_TYPE_None)		\
+	{										\
+		const struct spa_pod **d = va_arg(args, const struct spa_pod**);	\
+		if (d)									\
+			*d = (pod == NULL || (SPA_POD_TYPE(pod) == SPA_TYPE_None)	\
 				? NULL : pod);						\
 		break;									\
+	}										\
 	default:									\
 		break;									\
 	}										\
@@ -417,11 +425,11 @@ do {											\
 	case 'a':									\
 		va_arg(args, void*);							\
 		va_arg(args, void*);							\
-		/* fallthrough */							\
+		SPA_FALLTHROUGH 							\
 	case 'p':									\
 	case 'y':									\
 		va_arg(args, void*);							\
-		/* fallthrough */							\
+		SPA_FALLTHROUGH 							\
 	case 'b':									\
 	case 'I':									\
 	case 'i':									\
@@ -445,7 +453,7 @@ do {											\
 static inline int spa_pod_parser_getv(struct spa_pod_parser *parser, va_list args)
 {
 	struct spa_pod_frame *f = parser->state.frame;
-        uint32_t ftype = f ? f->pod.type : SPA_TYPE_Struct;
+        uint32_t ftype = f ? f->pod.type : (uint32_t)SPA_TYPE_Struct;
 	const struct spa_pod_prop *prop = NULL;
 	int count = 0;
 
@@ -563,6 +571,10 @@ static inline int spa_pod_parser_get(struct spa_pod_parser *parser, ...)
 	spa_pod_parser_get_struct(&_p,##__VA_ARGS__);		\
 })
 
+/**
+ * \}
+ */
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
