@@ -163,7 +163,7 @@ static void init_node(struct impl *this, struct node *node, uint32_t id)
 
 static const struct a2dp_codec *get_a2dp_codec(enum spa_bluetooth_audio_codec id)
 {
-	const struct a2dp_codec **c;
+	const struct a2dp_codec * const *c;
 
 	for (c = a2dp_codecs; *c; ++c)
 		if ((*c)->id == id)
@@ -506,8 +506,14 @@ static const struct spa_bt_transport_events dynamic_node_transport_events = {
 static void emit_dynamic_node(struct dynamic_node *this, struct impl *impl,
 	struct spa_bt_transport *t, uint32_t id, const char *factory_name)
 {
-	if (this->transport != NULL)
-		return;
+	spa_log_debug(impl->log, NAME": dynamic node, transport: %p->%p id: %08x->%08x",
+		this->transport, t, this->id, id);
+
+	if (this->transport) {
+		/* Session manager don't really handles transport ptr changing. */
+		spa_assert(this->transport == t);
+		spa_hook_remove(&this->transport_listener);
+	}
 
 	this->impl = impl;
 	this->transport = t;
@@ -678,7 +684,9 @@ static int set_profile(struct impl *this, uint32_t profile, enum spa_bluetooth_a
 	 */
 	if (profile == DEVICE_PROFILE_A2DP && !(this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)) {
 		int ret;
-		const struct a2dp_codec *codec_list[2], **codecs, *a2dp_codec;
+		const struct a2dp_codec *codec_list[2];
+		const struct a2dp_codec * const *codecs;
+		const struct a2dp_codec *a2dp_codec;
 
 		a2dp_codec = get_a2dp_codec(codec);
 		if (a2dp_codec == NULL) {
@@ -1705,7 +1713,7 @@ static int impl_set_param(void *object,
 	switch (id) {
 	case SPA_PARAM_Profile:
 	{
-		uint32_t id, next;
+		uint32_t idx, next;
 		uint32_t profile;
 		enum spa_bluetooth_audio_codec codec;
 
@@ -1714,13 +1722,13 @@ static int impl_set_param(void *object,
 
 		if ((res = spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamProfile, NULL,
-				SPA_PARAM_PROFILE_index, SPA_POD_Int(&id))) < 0) {
+				SPA_PARAM_PROFILE_index, SPA_POD_Int(&idx))) < 0) {
 			spa_log_warn(this->log, "can't parse profile");
 			spa_debug_pod(0, NULL, param);
 			return res;
 		}
 
-		profile = get_profile_from_index(this, id, &next, &codec);
+		profile = get_profile_from_index(this, idx, &next, &codec);
 		if (profile == SPA_ID_INVALID)
 			return -EINVAL;
 
@@ -1729,7 +1737,7 @@ static int impl_set_param(void *object,
 	}
 	case SPA_PARAM_Route:
 	{
-		uint32_t id, device;
+		uint32_t idx, device;
 		struct spa_pod *props = NULL;
 		struct node *node;
 		bool save = false;
@@ -1739,7 +1747,7 @@ static int impl_set_param(void *object,
 
 		if ((res = spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamRoute, NULL,
-				SPA_PARAM_ROUTE_index, SPA_POD_Int(&id),
+				SPA_PARAM_ROUTE_index, SPA_POD_Int(&idx),
 				SPA_PARAM_ROUTE_device, SPA_POD_Int(&device),
 				SPA_PARAM_ROUTE_props, SPA_POD_OPT_Pod(&props),
 				SPA_PARAM_ROUTE_save, SPA_POD_OPT_Bool(&save))) < 0) {
