@@ -33,9 +33,7 @@
  * </refsect2>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 #include "gstpipewiresink.h"
 
 #include <string.h>
@@ -71,7 +69,7 @@ enum
 GType
 gst_pipewire_sink_mode_get_type (void)
 {
-  static volatile gsize mode_type = 0;
+  static gsize mode_type = 0;
   static const GEnumValue mode[] = {
     {GST_PIPEWIRE_SINK_MODE_DEFAULT, "GST_PIPEWIRE_SINK_MODE_DEFAULT", "default"},
     {GST_PIPEWIRE_SINK_MODE_RENDER, "GST_PIPEWIRE_SINK_MODE_RENDER", "render"},
@@ -255,6 +253,10 @@ pool_activated (GstPipeWirePool *pool, GstPipeWireSink *sink)
 	      SPA_MAX(MIN_BUFFERS, min_buffers),
 	      max_buffers ? max_buffers : INT32_MAX),
       SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16),
+      SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(
+						(1<<SPA_DATA_DmaBuf) |
+						(1<<SPA_DATA_MemFd) |
+						(1<<SPA_DATA_MemPtr)),
       0);
   port_params[0] = spa_pod_builder_pop (&b, &f);
 
@@ -537,7 +539,7 @@ gst_pipewire_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     while (TRUE) {
       state = pw_stream_get_state (pwsink->stream, &error);
 
-      if (state == PW_STREAM_STATE_PAUSED)
+      if (state >= PW_STREAM_STATE_PAUSED)
         break;
 
       if (state == PW_STREAM_STATE_ERROR)
@@ -574,6 +576,7 @@ gst_pipewire_sink_render (GstBaseSink * bsink, GstBuffer * buffer)
   GstPipeWireSink *pwsink;
   GstFlowReturn res = GST_FLOW_OK;
   const char *error = NULL;
+  gboolean unref_buffer = FALSE;
 
   pwsink = GST_PIPEWIRE_SINK (bsink);
 
@@ -612,10 +615,11 @@ gst_pipewire_sink_render (GstBaseSink * bsink, GstBuffer * buffer)
       goto done;
 
     gst_buffer_map (b, &info, GST_MAP_WRITE);
-    gst_buffer_extract (buffer, 0, info.data, info.size);
+    gst_buffer_extract (buffer, 0, info.data, info.maxsize);
     gst_buffer_unmap (b, &info);
     gst_buffer_resize (b, 0, gst_buffer_get_size (buffer));
     buffer = b;
+    unref_buffer = TRUE;
 
     pw_thread_loop_lock (pwsink->core->loop);
     if (pw_stream_get_state (pwsink->stream, &error) != PW_STREAM_STATE_STREAMING)
@@ -624,6 +628,8 @@ gst_pipewire_sink_render (GstBaseSink * bsink, GstBuffer * buffer)
 
   GST_DEBUG ("push buffer");
   do_send_buffer (pwsink, buffer);
+  if (unref_buffer)
+    gst_buffer_unref (buffer);
 
 done_unlock:
   pw_thread_loop_unlock (pwsink->core->loop);
@@ -776,6 +782,7 @@ gst_pipewire_sink_change_state (GstElement * element, GstStateChange transition)
       pw_thread_loop_lock (this->core->loop);
       pw_stream_set_active(this->stream, true);
       pw_thread_loop_unlock (this->core->loop);
+      gst_buffer_pool_set_flushing(GST_BUFFER_POOL_CAST(this->pool), FALSE);
       break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       /* stop play ASAP by corking */
