@@ -41,6 +41,8 @@
 
 #define NAME "link"
 
+#define MAX_HOPS	32
+
 #define pw_link_resource_info(r,...)      pw_resource_call(r,struct pw_link_events,info,0,__VA_ARGS__)
 
 /** \cond */
@@ -461,7 +463,7 @@ static int do_allocation(struct pw_impl_link *this)
 						output->buffers.n_buffers)) < 0) {
 			error = spa_aprintf("error use output buffers: %d (%s)", res,
 					spa_strerror(res));
-			goto error;
+			goto error_clear;
 		}
 		if (SPA_RESULT_IS_ASYNC(res)) {
 			res = spa_node_sync(output->node->node, res),
@@ -494,8 +496,9 @@ static int do_allocation(struct pw_impl_link *this)
 	}
 	return 0;
 
-error:
+error_clear:
 	pw_buffers_clear(&output->buffers);
+error:
 	link_update_state(this, PW_LINK_STATE_ERROR, res, error);
 	return res;
 }
@@ -636,6 +639,8 @@ static void input_remove(struct pw_impl_link *this, struct pw_impl_port *port)
 	spa_list_remove(&this->input_link);
 	pw_impl_port_emit_link_removed(this->input, this);
 
+	pw_impl_port_recalc_latency(this->input);
+
 	if ((res = pw_impl_port_use_buffers(port, mix, 0, NULL, 0)) < 0) {
 		pw_log_warn(NAME" %p: port %p clear error %s", this, port, spa_strerror(res));
 	}
@@ -656,6 +661,8 @@ static void output_remove(struct pw_impl_link *this, struct pw_impl_port *port)
 	spa_list_remove(&this->output_link);
 	pw_impl_port_emit_link_removed(this->output, this);
 
+	pw_impl_port_recalc_latency(this->output);
+
 	/* we don't clear output buffers when the link goes away. They will get
 	 * cleared when the node goes to suspend */
 	pw_impl_port_release_mix(port, mix);
@@ -791,16 +798,15 @@ static void port_param_changed(struct pw_impl_link *this, uint32_t id,
 {
 	enum pw_impl_port_state target;
 
-	pw_log_debug(NAME" %p: outport %p input %p param %d", this,
-		outport, inport, id);
+	pw_log_debug(NAME" %p: outport %p input %p param %d (%s)", this,
+		outport, inport, id, spa_debug_type_find_name(spa_type_param, id));
 
 	switch (id) {
 	case SPA_PARAM_EnumFormat:
 		target = PW_IMPL_PORT_STATE_CONFIGURE;
 		break;
-//	case SPA_PARAM_Buffers:
-//		target = PW_IMPL_PORT_STATE_READY;
-//		break;
+	case SPA_PARAM_Latency:
+		return;
 	default:
 		return;
 	}
@@ -842,16 +848,34 @@ static void output_port_state_changed(void *data, enum pw_impl_port_state old,
 	port_state_changed(this, this->output, this->input, state, error);
 }
 
+static void input_port_latency_changed(void *data)
+{
+	struct impl *impl = data;
+	struct pw_impl_link *this = &impl->this;
+	if (!this->feedback)
+		pw_impl_port_recalc_latency(this->output);
+}
+
+static void output_port_latency_changed(void *data)
+{
+	struct impl *impl = data;
+	struct pw_impl_link *this = &impl->this;
+	if (!this->feedback)
+		pw_impl_port_recalc_latency(this->input);
+}
+
 static const struct pw_impl_port_events input_port_events = {
 	PW_VERSION_IMPL_PORT_EVENTS,
 	.param_changed = input_port_param_changed,
 	.state_changed = input_port_state_changed,
+	.latency_changed = input_port_latency_changed,
 };
 
 static const struct pw_impl_port_events output_port_events = {
 	PW_VERSION_IMPL_PORT_EVENTS,
 	.param_changed = output_port_param_changed,
 	.state_changed = output_port_state_changed,
+	.latency_changed = output_port_latency_changed,
 };
 
 static void node_result(struct impl *impl, struct pw_impl_port *port,
@@ -906,26 +930,30 @@ static const struct pw_impl_node_events output_node_events = {
 	.active_changed = node_active_changed,
 };
 
-static bool pw_impl_node_can_reach(struct pw_impl_node *output, struct pw_impl_node *input)
+static bool pw_impl_node_can_reach(struct pw_impl_node *output, struct pw_impl_node *input, int hop)
 {
 	struct pw_impl_port *p;
+	struct pw_impl_link *l;
+
+	output->loopchecked = true;
 
 	if (output == input)
 		return true;
 
-	spa_list_for_each(p, &output->output_ports, link) {
-		struct pw_impl_link *l;
+	if (hop == MAX_HOPS) {
+		pw_log_warn("exceeded hops (%d) %s -> %s", hop, output->name, input->name);
+		return false;
+	}
 
+	spa_list_for_each(p, &output->output_ports, link) {
+		spa_list_for_each(l, &p->links, output_link)
+			l->input->node->loopchecked = l->feedback;
+	}
+	spa_list_for_each(p, &output->output_ports, link) {
 		spa_list_for_each(l, &p->links, output_link) {
-			if (l->feedback)
+			if (l->input->node->loopchecked)
 				continue;
-			if (l->input->node == input)
-				return true;
-		}
-		spa_list_for_each(l, &p->links, output_link) {
-			if (l->feedback)
-				continue;
-			if (pw_impl_node_can_reach(l->input->node, input))
+			if (pw_impl_node_can_reach(l->input->node, input, hop+1))
 				return true;
 		}
 	}
@@ -1069,15 +1097,17 @@ struct pw_impl_link *pw_context_create_link(struct pw_context *context,
 		goto error_no_mem;
 
 	this = &impl->this;
-	this->feedback = pw_impl_node_can_reach(input_node, output_node);
+	this->feedback = pw_impl_node_can_reach(input_node, output_node, 0);
 	pw_properties_set(properties, PW_KEY_LINK_FEEDBACK, this->feedback ? "true" : NULL);
 
 	pw_log_debug(NAME" %p: new out-port:%p -> in-port:%p", this, output, input);
 
 	if (user_data_size > 0)
-                this->user_data = SPA_MEMBER(impl, sizeof(struct impl), void);
+                this->user_data = SPA_PTROFF(impl, sizeof(struct impl), void);
 
 	impl->work = pw_work_queue_new(context->main_loop);
+	if (impl->work == NULL)
+		goto error_work_queue;
 
 	this->context = context;
 	this->properties = properties;
@@ -1147,6 +1177,9 @@ struct pw_impl_link *pw_context_create_link(struct pw_context *context,
 
 	try_link_controls(impl, output, input);
 
+	pw_impl_port_recalc_latency(this->output);
+	pw_impl_port_recalc_latency(this->input);
+
 	pw_impl_node_emit_peer_added(impl->onode, impl->inode);
 
 	return this;
@@ -1171,14 +1204,17 @@ error_no_mem:
 	res = -errno;
 	pw_log_debug("alloc failed: %m");
 	goto error_exit;
+error_work_queue:
+	res = -errno;
+	pw_log_debug("work queue failed: %m");
+	goto error_free;
 error_no_io:
 	pw_log_debug(NAME" %p: can't set io %d (%s)", this, res, spa_strerror(res));
 	goto error_free;
 error_free:
 	free(impl);
 error_exit:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
 	errno = -res;
 	return NULL;
 }
@@ -1200,9 +1236,7 @@ SPA_EXPORT
 int pw_impl_link_register(struct pw_impl_link *link,
 		     struct pw_properties *properties)
 {
-	struct pw_context *context = link->context;
-	struct pw_impl_node *output_node, *input_node;
-	const char *keys[] = {
+	static const char * const keys[] = {
 		PW_KEY_OBJECT_PATH,
 		PW_KEY_MODULE_ID,
 		PW_KEY_FACTORY_ID,
@@ -1214,6 +1248,9 @@ int pw_impl_link_register(struct pw_impl_link *link,
 		NULL
 	};
 
+	struct pw_context *context = link->context;
+	struct pw_impl_node *output_node, *input_node;
+
 	if (link->registered)
 		goto error_existed;
 
@@ -1255,8 +1292,7 @@ int pw_impl_link_register(struct pw_impl_link *link,
 	return 0;
 
 error_existed:
-	if (properties)
-		pw_properties_free(properties);
+	pw_properties_free(properties);
 	return -EEXIST;
 }
 
